//===========================================================================
//
//      s_expm1f.c
//
//      Part of the standard mathematical function library
//
//===========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//===========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):   luoqi
// Contributors:  luoqi
// Date:        2001-09-20
// Purpose:     
// Description: 
// Usage:       
//
//####DESCRIPTIONEND####
//
//===========================================================================

// CONFIGURATION

#include <pkgconf/libm.h>   // Configuration header

// Include the Math library?
#ifdef CYGPKG_LIBM     

// Derived from code with the following copyright


/* @(#)s_expm1.c 1.3 95/01/18 */
/*
 * ====================================================
 * Copyright (C) 1993 by Sun Microsystems, Inc. All rights reserved.
 *
 * Developed at SunSoft, a Sun Microsystems, Inc. business.
 * Permission to use, copy, modify, and distribute this
 * software is freely granted, provided that this notice 
 * is preserved.
 * ====================================================
 */

/* expm1(x)
 * Returns exp(x)-1, the exponential of x minus 1.
 *
 * Method
 *   1. Argument reduction:
 *      Given x, find r and integer k such that
 *
 *               x = k*ln2 + r,  |r| <= 0.5*ln2 ~ 0.34658  
 *
 *      Here a correction term c will be computed to compensate 
 *      the error in r when rounded to a floating-point number.
 *
 *   2. Approximating expm1(r) by a special rational function on
 *      the interval [0,0.34658]:
 *      Since
 *          r*(exp(r)+1)/(exp(r)-1) = 2+ r^2/6 - r^4/360 + ...
 *      we define R1(r*r) by
 *          r*(exp(r)+1)/(exp(r)-1) = 2+ r^2/6 * R1(r*r)
 *      That is,
 *          R1(r**2) = 6/r *((exp(r)+1)/(exp(r)-1) - 2/r)
 *                   = 6/r * ( 1 + 2.0*(1/(exp(r)-1) - 1/r))
 *                   = 1 - r^2/60 + r^4/2520 - r^6/100800 + ...
 *      We use a special Reme algorithm on [0,0.347] to generate 
 *      a polynomial of degree 5 in r*r to approximate R1. The 
 *      maximum error of this polynomial approximation is bounded 
 *      by 2**-61. In other words,
 *          R1(z) ~ 1.0 + Q1*z + Q2*z**2 + Q3*z**3 + Q4*z**4 + Q5*z**5
 *      where   Q1  =  -1.6666666666666567384E-2,
 *              Q2  =   3.9682539681370365873E-4,
 *              Q3  =  -9.9206344733435987357E-6,
 *              Q4  =   2.5051361420808517002E-7,
 *              Q5  =  -6.2843505682382617102E-9;
 *      (where z=r*r, and the values of Q1 to Q5 are listed below)
 *      with error bounded by
 *          |                  5           |     -61
 *          | 1.0+Q1*z+...+Q5*z   -  R1(z) | <= 2 
 *          |                              |
 *      
 *      expm1(r) = exp(r)-1 is then computed by the following 
 *      specific way which minimize the accumulation rounding error: 
 *                             2     3
 *                            r     r    [ 3 - (R1 + R1*r/2)  ]
 *            expm1(r) = r + --- + --- * [--------------------]
 *                            2     2    [ 6 - r*(3 - R1*r/2) ]
 *      
 *      To compensate the error in the argument reduction, we use
 *              expm1(r+c) = expm1(r) + c + expm1(r)*c 
 *                         ~ expm1(r) + c + r*c 
 *      Thus c+r*c will be added in as the correction terms for
 *      expm1(r+c). Now rearrange the term to avoid optimization 
 *      screw up:
 *                      (      2                                    2 )
 *                      ({  ( r    [ R1 -  (3 - R1*r/2) ]  )  }    r  )
 *       expm1(r+c)~r - ({r*(--- * [--------------------]-c)-c} - --- )
 *                      ({  ( 2    [ 6 - r*(3 - R1*r/2) ]  )  }    2  )
 *                      (                                             )
 *      
 *                 = r - E
 *   3. Scale back to obtain expm1(x):
 *      From step 1, we have
 *         expm1(x) = either 2^k*[expm1(r)+1] - 1
 *                  = or     2^k*[expm1(r) + (1-2^-k)]
 *   4. Implementation notes:
 *      (A). To save one multiplication, we scale the coefficient Qi
 *           to Qi*2^i, and replace z by (x^2)/2.
 *      (B). To achieve maximum accuracy, we compute expm1(x) by
 *        (i)   if x < -56*ln2, return -1.0, (raise inexact if x!=inf)
 *        (ii)  if k=0, return r-E
 *        (iii) if k=-1, return 0.5*(r-E)-0.5
 *        (iv)  if k=1 if r < -0.25, return 2*((r+0.5)- E)
 *                     else          return  1.0+2.0*(r-E);
 *        (v)   if (k<-2||k>56) return 2^k(1-(E-r)) - 1 (or exp(x)-1)
 *        (vi)  if k <= 20, return 2^k((1-2^-k)-(E-r)), else
 *        (vii) return 2^k(1-((E+2^-k)-r)) 
 *
 * Special cases:
 *      expm1(INF) is INF, expm1(NaN) is NaN;
 *      expm1(-INF) is -1, and
 *      for finite argument, only expm1(0)=0 is exact.
 *
 * Accuracy:
 *      according to an error analysis, the error is always less than
 *      1 ulp (unit in the last place).
 *
 * Misc. info.
 *      For IEEE double 
 *          if x >  7.09782712893383973096e+02 then expm1(x) overflow
 *
 * Constants:
 * The hexadecimal values are the intended ones for the following 
 * constants. The decimal values may be used, provided that the 
 * compiler will convert from decimal to binary accurately enough
 * to produce the hexadecimal values shown.
 */

#include "mathincl/fdlibm.h"

static const float
one             = 1.0,
huge            = 1.0e+30,
tiny            = 1.0e-30,
o_threshold     = 8.8721679688e+01,/* 0x42b17180 */
ln2_hi          = 6.9313812256e-01,/* 0x3f317180 */
ln2_lo          = 9.0580006145e-06,/* 0x3717f7d1 */
invln2          = 1.4426950216e+00,/* 0x3fb8aa3b */
        /* scaled coefficients related to expm1 */
Q1  =  -3.3333335072e-02, /* 0xbd088889 */
Q2  =   1.5873016091e-03, /* 0x3ad00d01 */
Q3  =  -7.9365076090e-05, /* 0xb8a670cd */
Q4  =   4.0082177293e-06, /* 0x36867e54 */
Q5  =  -2.0109921195e-07; /* 0xb457edbb */

        float expm1f(float x)
{
        float y,hi,lo,c,t,e,hxs,hfx,r1;
        int k,xsb;
        unsigned hx;

        c=0.0;                          /* placate compiler */
        hx  = CYG_LIBM_FL(x);
        xsb = hx&0x80000000;            /* sign bit of x */
        if(xsb==0) y=x; else y= -x;     /* y = |x| */
        hx &= 0x7fffffff;               /* high word of |x| */

    /* filter out huge and non-finite argument */
        if(hx >= 0x4195b844) {                  /* if |x|>=27*ln2 */
            if(hx >= 0x42b17218) {              /* if |x|>=88.721... */
                if(hx>0x7f800000)
		    return x+x;     /* NaN */
                if(hx==0x7f800000)
                    return (xsb==0)? x:-1.0;/* exp(+-inf)={inf,-1} */
                if(x > o_threshold) return huge*huge; /* overflow */
            }
            if(xsb!=0) { /* x < -27*ln2, return -1.0 with inexact */
                if(x+tiny<(float)0.0)          /* raise inexact */
                return tiny-one;        /* return -1 */
            }
        }

    /* argument reduction */
        if(hx > 0x3eb17218) {           /* if  |x| > 0.5 ln2 */ 
            if(hx < 0x3f851592) {       /* and |x| < 1.5 ln2 */
                if(xsb==0)
                    {hi = x - ln2_hi; lo =  ln2_lo;  k =  1;}
                else
                    {hi = x + ln2_hi; lo = -ln2_lo;  k = -1;}
            } else {
                k  = invln2*x+((xsb==0)?(float)0.5:(float)-0.5);
                t  = k;
                hi = x - t*ln2_hi;      /* t*ln2_hi is exact here */
                lo = t*ln2_lo;
            }
            x  = hi - lo;
            c  = (hi-x)-lo;
        } 
        else if(hx < 0x33000000) {      /* when |x|<2**-25, return x */
            t = huge+x; /* return x with inexact flags when x!=0 */
            return x - (t-(huge+x));    
        }
        else k = 0;

    /* x is now in primary range */
        hfx = (float)0.5*x;
        hxs = x*hfx;
        r1 = one+hxs*(Q1+hxs*(Q2+hxs*(Q3+hxs*(Q4+hxs*Q5))));
        t  = (float)3.0-r1*hfx;
        e  = hxs*((r1-t)/((float)6.0 - x*t));
        if(k==0) return x - (x*e-hxs);          /* c is 0 */
        else {
            e  = (x*(e-c)-c);
            e -= hxs;
            if(k== -1) return (float)0.5*(x-e)-(float)0.5;
            if(k==1) {
                if(x < (float)-0.25) return -(float)2.0*(e-(x+(float)0.5));
                else          return  one+(float)2.0*(x-e);
            }
            if (k <= -2 || k>56) {   /* suffice to return exp(x)-1 */
                y = one-(e-x);
                CYG_LIBM_FL(y) += (k<<23);      /* add k to y's exponent */
                return y-one;
            }
            t = one;
            if(k<23) {
                CYG_LIBM_FL(t) = 0x3f800000 - (0x1000000>>k);  /* t=1-2^-k */
                y = t-(e-x);
                CYG_LIBM_FL(y) += (k<<23);      /* add k to y's exponent */
           } else {
                CYG_LIBM_FL(t)  = ((0x7f-k)<<23);      /* 2^-k */
                y = x-(e+t);
                y += one;
                CYG_LIBM_FL(y) += (k<<23);      /* add k to y's exponent */
            }
        }
        return y;
}

#endif // ifdef CYGPKG_LIBM     

// EOF s_expm1f.c
